import http.server
import os
import socketserver
import threading


class HTTPServer:
    def __init__(self, ip, port, directory):
        self.handler = http.server.SimpleHTTPRequestHandler
        self.directory = directory
        self.prev_directory = os.getcwd()
        self.server = socketserver.TCPServer((ip, port), self.handler)
        self.server_thread = threading.Thread(
            target=self.server.serve_forever,
            daemon=True,
        )

    def start(self):
        print("Starting HTTP server...", end=" ", flush=True)
        os.chdir(self.directory)
        self.server_thread.start()
        print("✓")

    def stop(self):
        print("Stoping HTTP server...", end=" ", flush=True)
        self.server.shutdown()
        os.chdir(self.prev_directory)
        print("✓")



import http.client
import json


class MorningstreamsClient:
    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.client = None
        self.token = None

    def __enter__(self):
        self.client = http.client.HTTPSConnection("api.morningstreams.com")
        self.login()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.client.close()

    def login(self):
        print(f"Logging in as {self.username}...", end=" ", flush=True)
        path = "/api/auth/login"
        credentials = {"username": self.username, "password": self.password}
        headers = {"Content-type": "application/json"}
        self.client.request("POST", path, json.dumps(credentials), headers)
        response = json.loads(self.client.getresponse().read().decode("utf-8"))
        if "token" in response:
            self.token = response["token"]
            print("✓")
        else:
            print("✗")
            raise http.client.HTTPException(response)

    def get_streams(self):
        print("Getting acestreams...", end=" ", flush=True)
        assert self.token is not None
        path = "/api/acestreams"
        headers = {"authorization": f"bearer {self.token}"}
        self.client.request("GET", path, headers=headers)
        streams = json.loads(self.client.getresponse().read().decode("utf-8"))
        print(f"found {len(streams)}")
        for stream in streams:
            print(f"  - {stream['title']} 👍 {stream['likesCount']}")
        return streams

    def generate_m3u8(self, ip):
        s = "#EXTM3U\n"
        for link in self.get_streams():
            try:
                int(link["contentId"], 16)  # check if is a acestream link
                s += f'#EXTINF:-1,{link["title"]}\n'
                s += f'http://{ip}:6878/ace/getstream?id={link["contentId"]}\n'
            except ValueError:
                pass
        return s



import subprocess
import time
import urllib.error
import urllib.request

import click


class AcestreamEngine:
    def __init__(self, start_cmd, stop_cmd, timeout=10):
        self.start_cmd = start_cmd
        self.stop_cmd = stop_cmd
        self.timeout = timeout

    def start(self):
        if self.is_running:
            click.echo("Another instance of Acestream engine is running.")
            if click.confirm("Would you like to restart it?"):
                self.stop()
                self.start()
            return
        print("Starting Acestream engine...", end=" ", flush=True)
        start_time = time.time()
        subprocess.Popen(self.start_cmd, stdout=subprocess.DEVNULL)
        while time.time() - start_time < self.timeout:
            if self.is_running:
                print("✓")
                return
            time.sleep(1)
        print("✗")
        raise EnvironmentError("Cannot start acestream engine.")

    def stop(self):
        print("Stoping Acestream engine...", end=" ", flush=True)
        if not self.is_running:
            print("✓")
            return
        start_time = time.time()
        process = subprocess.Popen(self.stop_cmd, stdout=subprocess.DEVNULL)
        process.wait()
        while time.time() - start_time < self.timeout:
            if not self.is_running:
                print("✓")
                return
            time.sleep(1)
        print("✗")
        raise EnvironmentError("Cannot stop acestream engine.")

    @property
    def is_running(self):
        url = "http://127.0.0.1:6878/webui/api/service?method=get_version"
        try:
            urllib.request.urlopen(url)
            return True
        except (urllib.error.URLError, ConnectionResetError):
            return False



import pathlib
import platform

import click

from . import utils

repo = pathlib.Path(__file__).parent.parent
path_credentials = repo / "credentials.txt"


@click.group()
def cli():
    """
    \b
    Expose acestreams from morningstreams in you local network.
    Source code: https://github.com/S1M0N38/morningstreams
    """
    pass


@cli.command(help="Install acestream engine")
def install():
    """This script prints hello NAME COUNT times."""
    if utils.is_raspberrypi():
        utils.installer_rpi()
        click.secho("Acestream engine installed.", fg="green", bold=True)
    elif utils.is_macos():
        msg = "Is docker (https://docs.docker.com) running in the backgound?"
        if click.confirm(msg):
            utils.installer_macos()
            click.secho("Acestream engine installed.", fg="green", bold=True)
    else:
        msg = (
            f"{platform.system()} - {platform.machine()} is not supported. "
            "You can still run the script with --no-engine flag."
        )
        raise EnvironmentError(msg)


@cli.command(help="Login and store credentials for furture logins")
@click.option("--username", prompt=True, envvar="MORNINGSTREAMS_USERNAME")
@click.password_option(envvar="MORNINGSTREAMS_PASSWORD")
def login(username, password):
    if utils.valid_credentials(username, password):
        with open(path_credentials, "w") as f:
            f.write(f"{username}\n{password}")
        click.secho("Sucessfully logged in.", fg="green", bold=True)
    else:
        click.secho("Invalid credendial.", fg="red", bold=True)


@cli.command(help="Show login status")
def status():
    if path_credentials.exists():
        with open(path_credentials) as f:
            username = f.readline().strip()
        click.echo(f"Your are logged in as {username}.")
    else:
        click.echo("Your are not logged in.")


@cli.command(help="Delete login credentials if exists")
def logout():
    if path_credentials.exists():
        path_credentials.unlink()
        click.echo("Sucessfully logged out.")
    else:
        click.echo("Your are not logged in.")


@cli.command(help="Expose streams on your local network")
@click.option(
    "--ip",
    default="127.0.0.1",
    show_default=True,
    help="Ip where playlist is exposed.",
)
@click.option(
    "--port",
    type=int,
    default=8080,
    show_default=True,
    help="Port where playlist is exposed.",
)
@click.option(
    "--server/--no-server",
    default=True,
    show_default=True,
    help="Expose playlist.m3u8.",
)
@click.option(
    "--engine/--no-engine",
    default=True,
    show_default=True,
    help="Start acestream engine.",
)
def run(**args):
    if not path_credentials.exists():
        msg = "You have to be logged in. Use morningstream login"
        click.secho(msg, fg="yellow")
        return
    with open(path_credentials) as f:
        username = f.readline().strip()
        password = f.readline().strip()
    utils.run(username, password, args)


if __name__ == "__main__":
    cli()



import hashlib
import http.client
import io
import pathlib
import platform
import subprocess
import tarfile
import time
import urllib.request

import click

from .core.client import MorningstreamsClient
from .core.engine import AcestreamEngine
from .core.server import HTTPServer

# paths #######################################################################

repo = pathlib.Path(__file__).parent.parent
path_playlist = repo / "playlist.m3u8"

# platforms ###################################################################


def is_raspberrypi():
    try:
        with io.open("/sys/firmware/devicetree/base/model", "r") as m:
            if "raspberry pi" in m.read().lower():
                return True
    except Exception:
        pass
    return False


def is_macos():
    # TODO maybe check for architecture: intel vs arm
    if platform.system() == "Darwin":
        return True
    return False


# install #####################################################################


def installer_rpi():

    # Download acestream-engine for Rasberry PI
    version = "acestream_3.1.48_Py2.7.16%2B_LinaroNDK_webUI_ARMv7.tar.gz"
    url = (
        "https://github.com/moromete/repository.moromete.addons/"
        "raw/master/plugin.video.streams/" + version
    )
    sha265 = "a4a73f84f33139ec5c0decdf1c1edc1a9d83737eac3ce6df2005e1099826e297"
    path_tar_gz = repo / "acestream.tar.gz"
    path_acestream_engine = repo / "acestream.engine"

    print("Removing old acestream engine if exists...", end=" ", flush=True)
    path_tar_gz.unlink(missing_ok=True)
    subprocess.run(
        ["sudo", "rm", "-rf", path_acestream_engine],
        stderr=subprocess.DEVNULL,
        stdout=subprocess.DEVNULL,
    )
    if path_acestream_engine.exists():
        print("✗")
        msg = "You have to reboot the Raspberry Pi (sudo reboot) and try again"
        raise PermissionError(msg)
    print("✓")

    with open(path_tar_gz, "wb") as f:
        with urllib.request.urlopen(url) as resp:
            length = int(resp.getheader("content-length"))
            with click.progressbar(
                length=length,
                label="Downloading acestream.tar.gz...",
                bar_template="%(label)s %(info)s",
            ) as bar:
                for _ in range(101):
                    chunk = resp.read(length // 100)
                    f.write(chunk)
                    bar.update(len(chunk))

    print("Checking SHA256 for acestream.tar.gz...", end=" ", flush=True)
    hash = hashlib.sha256()
    with open(path_tar_gz, "rb") as f:
        for block in iter(lambda: f.read(4096), b""):
            hash.update(block)
    assert sha265 == hash.hexdigest()
    print("✓")

    print("Extracting to acestream.engine...", end=" ", flush=True)
    tar = tarfile.open(path_tar_gz, "r:gz")
    tar.extractall(path=repo)
    tar.close()
    print("✓")

    print("Removing acestream.tar.gz...", end=" ", flush=True)
    path_tar_gz.unlink()
    print("✓")


def installer_macos():

    # Download and install acestream-engine for MacOs
    image = "magnetikonline/acestream-server"
    print("Removing old acestream engine if exists...")
    subprocess.run(["docker", "image", "rm", image])
    print("Install acestream engine...")
    subprocess.run(["docker", "pull", image])


# login #######################################################################


def valid_credentials(username, password):
    try:
        with MorningstreamsClient(username, password):
            return True
    except http.client.HTTPException:
        return False


# run #########################################################################


def _engine_commands():

    if is_raspberrypi():
        path_acestream_engine = repo / "acestream.engine"
        return (
            ["sudo", path_acestream_engine / "acestream.start"],
            ["sudo", path_acestream_engine / "acestream.stop"],
        )

    elif is_macos():
        return (
            [
                "docker",
                "run",
                "--platform",
                "linux/amd64",
                "--pull",
                "never",
                "--name",
                "acestream",
                "--publish",
                "6878:6878",
                "--rm",
                "--tmpfs",
                "/dev/disk/by-id:noexec,rw,size=4k",
                "magnetikonline/acestream-server",
            ],
            ["docker", "container", "kill", "acestream"],
        )
    else:
        msg = (
            f"{platform.system()} - {platform.machine()} is not supported to "
            "run acestream engine. You can still run the script with "
            "--no-engine flag."
        )
        raise EnvironmentError(msg)


def run(username, password, args):

    # Remove previous playlist
    path_playlist.unlink(missing_ok=True)

    # Create and start HTTP server
    if args["server"]:
        server = HTTPServer(args["ip"], args["port"], repo)
        server.start()

    # Create and start Acestream engine
    if args["engine"]:
        start_engine_cmd, stop_engine_cmd = _engine_commands()
        engine = AcestreamEngine(start_engine_cmd, stop_engine_cmd)
        engine.start()

    # Get streams and save into playlist.m3u8
    with MorningstreamsClient(username, password) as ms:
        with open(path_playlist, "w") as f:
            f.write(ms.generate_m3u8(args["ip"]))

    try:
        if args["server"]:
            address = f"http://{args['ip']}:{args['port']}/playlist.m3u8"
            click.secho(f"Exposing streams at {address}", fg="blue", bold=True)

        # TODO instead of this time.sleep, perform diagnostic about
        # the running stream and reload if it's stuck
        time.sleep(60 * 60 * 4)

    except KeyboardInterrupt:
        print()
        if args["server"]:
            server.stop()
        if args["engine"]:
            engine.stop()
